#genetic algorithm selection routines
#based on galib. 
#exception - these classes only work on the scaled fitness
from ga_util import *
import scipy.stats as rv
stats = rv
import pdb
from Numeric import *
from scipy_base.fastumath import *

class selector:
	def update(self,pop): pass
	def select(self,pop): raise GAError, 'selector.select() must be overridden'
	def clear(self): pass
class uniform_selector(selector):
	def select(self,pop,cnt = 1):
		if cnt == 1: return rv.choice(pop)
		res = []
		for i in range(cnt): res.append(rv.choice(pop))
		return res

class rank_selector(selector):
	def select(self,pop,cnt = 1):
		pop.sort()
		studliest = pop[0].fitness()
		tied_for_first = filter(lambda x,y=studliest: x.fitness()==y,pop)
		if cnt == 1: return rv.choice(tied_for_first)
		res = []
		for i in range(cnt): res.append(rv.choice(tied_for_first))
		return res

#scores must all be positive		
class roulette_selector(selector):
	def update(self,pop):
		self.pop = pop[:]
		sz = len(pop)
		if not sz: raise GAError, 'srs_selector - the pop size is 0!'
		f =self.pop.fitnesses()
		f_max = max(f); f_min = min(f)
		if not ( (f_max >= 0 and f_min >= 0) or 
			   (f_max <= 0 and f_min <= 0)):
			   raise GAError, 'srs_selector requires all fitnesses values to be either strictly positive or strictly negative'
		if f_max == f_min: f = ones(shape(f),typecode = Float32)
		self.dart_board = add.accumulate(f / sum(f))
	def select(self,pop,cnt = 1):
		returns = []
		for i in range(cnt):
			dart = rv.random()
			idx = 0
			#binary search would be faster
			while dart > self.dart_board[idx]: idx = idx + 1
			returns.append(self.pop[idx])
		if cnt == 1: return returns[0]
		else: return returns	
	def clear(self): 
		del self.pop
#scores must all be positive		
class srs_selector(selector):
	def update(self,pop):
		sz = len(pop)
		if not sz: raise GAError, 'srs_selector - the pop size is 0!'
		f =pop.fitnesses()
		f_max = max(f); f_min = min(f)
		if not ( (f_max >= 0. and f_min >= 0.) or 
			   (f_max <= 0. and f_min <= 0.)):
			   raise GAError, 'srs_selector requires all fitnesses values to be either strictly positive or strictly negative - min %f, max %f' %(f_min,f_max)
		f_avg = sum(f)/sz
		if f_avg == 0.: e = ones(shape(f),typecode = Float32)
		else:
			if pop.min_or_max() == 'max': e = f/f_avg
			else: e = (-f+f_max+f_min)/f_avg
		self.expected_value = e
		garauntee,chance = divmod(e,1.)
#		garauntee = floor(e)
#		chance = remainder(e,1)
		choices = []
		for i in xrange(sz): choices = choices + [pop[i]] * int(garauntee[i])
		#now deal with the remainder
		dart_board = add.accumulate(chance / sum(chance))
		for i in range(len(choices),sz):
			 dart = rv.random()
			 idx = 0
			 while dart > dart_board[idx]: idx = idx + 1
			 choices.append(pop[idx])
		self.choices = choices			 
	def select(self,pop,cnt = 1): #ignore the past in pop
		res = []
		for i in range(cnt): res.append(rv.choice(self.choices))
#		for chosen in res: self.choices.remove(chosen)
		if cnt == 1: return res[0]
		return res	
	def clear(self): 
		if hasattr(self,'choices'):
			del self.choices		



import unittest

import sys
from scipy_test.testing import *
set_package_path()
import scipy_base;reload(scipy_base)
from scipy_base import *
del sys.path[0]


class test_array_split(unittest.TestCase):
    def check_integer_0_split(self):
        a = arange(10)
        try:
            res = array_split(a,0)
            assert(0) # it should have thrown a value error
        except ValueError:
            pass
    def check_integer_split(self):
        a = arange(10)
        res = array_split(a,1)
        desired = [arange(10)]
        compare_results(res,desired)

        res = array_split(a,2)
        desired = [arange(5),arange(5,10)]
        compare_results(res,desired)

        res = array_split(a,3)
        desired = [arange(4),arange(4,7),arange(7,10)]
        compare_results(res,desired)

        res = array_split(a,4)
        desired = [arange(3),arange(3,6),arange(6,8),arange(8,10)]
        compare_results(res,desired)

        res = array_split(a,5)
        desired = [arange(2),arange(2,4),arange(4,6),arange(6,8),arange(8,10)]
        compare_results(res,desired)

        res = array_split(a,6)
        desired = [arange(2),arange(2,4),arange(4,6),arange(6,8),arange(8,9),
                   arange(9,10)]
        compare_results(res,desired)

        res = array_split(a,7)
        desired = [arange(2),arange(2,4),arange(4,6),arange(6,7),arange(7,8),
                   arange(8,9), arange(9,10)]
        compare_results(res,desired)

        res = array_split(a,8)
        desired = [arange(2),arange(2,4),arange(4,5),arange(5,6),arange(6,7),
                   arange(7,8), arange(8,9), arange(9,10)]
        compare_results(res,desired)

        res = array_split(a,9)
        desired = [arange(2),arange(2,3),arange(3,4),arange(4,5),arange(5,6),
                   arange(6,7), arange(7,8), arange(8,9), arange(9,10)]
        compare_results(res,desired)

        res = array_split(a,10)
        desired = [arange(1),arange(1,2),arange(2,3),arange(3,4),
                   arange(4,5),arange(5,6), arange(6,7), arange(7,8),
                   arange(8,9), arange(9,10)]
        compare_results(res,desired)

        res = array_split(a,11)
        desired = [arange(1),arange(1,2),arange(2,3),arange(3,4),
                   arange(4,5),arange(5,6), arange(6,7), arange(7,8),
                   arange(8,9), arange(9,10),array([])]
        compare_results(res,desired)
    def check_integer_split_2D_rows(self):
        a = array([arange(10),arange(10)])
        res = array_split(a,3,axis=0)
        desired = [array([arange(10)]),array([arange(10)]),array([])]
        compare_results(res,desired)
    def check_integer_split_2D_cols(self):
        a = array([arange(10),arange(10)])
        res = array_split(a,3,axis=-1)
        desired = [array([arange(4),arange(4)]),
                   array([arange(4,7),arange(4,7)]),
                   array([arange(7,10),arange(7,10)])]
        compare_results(res,desired)
    def check_integer_split_2D_default(self):
        """ This will fail if we change default axis
        """
        a = array([arange(10),arange(10)])
        res = array_split(a,3)
        desired = [array([arange(10)]),array([arange(10)]),array([])]
        compare_results(res,desired)
    #perhaps should check higher dimensions

    def check_index_split_simple(self):
        a = arange(10)
        indices = [1,5,7]
        res = array_split(a,indices,axis=-1)
        desired = [arange(0,1),arange(1,5),arange(5,7),arange(7,10)]
        compare_results(res,desired)

    def check_index_split_low_bound(self):
        a = arange(10)
        indices = [0,5,7]
        res = array_split(a,indices,axis=-1)
        desired = [array([]),arange(0,5),arange(5,7),arange(7,10)]
        compare_results(res,desired)
    def check_index_split_high_bound(self):
        a = arange(10)
        indices = [0,5,7,10,12]
        res = array_split(a,indices,axis=-1)
        desired = [array([]),arange(0,5),arange(5,7),arange(7,10),
                   array([]),array([])]
        compare_results(res,desired)
        
class test_split(unittest.TestCase):
    """* This function is essentially the same as array_split,
         except that it test if splitting will result in an
         equal split.  Only test for this case.
    *"""
    def check_equal_split(self):
        a = arange(10)
        res = split(a,2)
        desired = [arange(5),arange(5,10)]
        compare_results(res,desired)

    def check_unequal_split(self):
        a = arange(10) 
        try:
            res = split(a,3)
            assert(0) # should raise an error
        except ValueError:
            pass

class test_atleast_1d(unittest.TestCase):
    def check_0D_array(self):
        a = array(1); b = array(2);
        res=map(atleast_1d,[a,b])
        desired = [array([1]),array([2])]
        assert_array_equal(res,desired)
    def check_1D_array(self):
        a = array([1,2]); b = array([2,3]);
        res=map(atleast_1d,[a,b])
        desired = [array([1,2]),array([2,3])]
        assert_array_equal(res,desired)
    def check_2D_array(self):
        a = array([[1,2],[1,2]]); b = array([[2,3],[2,3]]);
        res=map(atleast_1d,[a,b])
        desired = [a,b]
        assert_array_equal(res,desired)
    def check_3D_array(self):
        a = array([[1,2],[1,2]]); b = array([[2,3],[2,3]]);
        a = array([a,a]);b = array([b,b]);
        res=map(atleast_1d,[a,b])
        desired = [a,b]
        assert_array_equal(res,desired)
    def check_r1array(self):
        """ Test to make sure equivalent Travis O's r1array function
        """
        assert(atleast_1d(3).shape == (1,))
        assert(atleast_1d(3j).shape == (1,))
        assert(atleast_1d(3L).shape == (1,))
        assert(atleast_1d(3.0).shape == (1,))
        assert(atleast_1d([[2,3],[4,5]]).shape == (2,2))

class test_atleast_2d(unittest.TestCase):
    def check_0D_array(self):
        a = array(1); b = array(2);
        res=map(atleast_2d,[a,b])
        desired = [array([[1]]),array([[2]])]
        assert_array_equal(res,desired)
    def check_1D_array(self):
        a = array([1,2]); b = array([2,3]);
        res=map(atleast_2d,[a,b])
        desired = [array([[1,2]]),array([[2,3]])]
        assert_array_equal(res,desired)
    def check_2D_array(self):
        a = array([[1,2],[1,2]]); b = array([[2,3],[2,3]]);
        res=map(atleast_2d,[a,b])
        desired = [a,b]
        assert_array_equal(res,desired)
    def check_3D_array(self):
        a = array([[1,2],[1,2]]); b = array([[2,3],[2,3]]);
        a = array([a,a]);b = array([b,b]);
        res=map(atleast_2d,[a,b])
        desired = [a,b]
        assert_array_equal(res,desired)
    def check_r2array(self):
        """ Test to make sure equivalent Travis O's r2array function
        """
        assert(atleast_2d(3).shape == (1,1))
        assert(atleast_2d([3j,1]).shape == (1,2))
        assert(atleast_2d([[[3,1],[4,5]],[[3,5],[1,2]]]).shape == (2,2,2))

class test_atleast_3d(unittest.TestCase):
    def check_0D_array(self):
        a = array(1); b = array(2);
        res=map(atleast_3d,[a,b])
        desired = [array([[[1]]]),array([[[2]]])]
        assert_array_equal(res,desired)
    def check_1D_array(self):
        a = array([1,2]); b = array([2,3]);
        res=map(atleast_3d,[a,b])
        desired = [array([[[1],[2]]]),array([[[2],[3]]])]
        assert_array_equal(res,desired)
    def check_2D_array(self):
        a = array([[1,2],[1,2]]); b = array([[2,3],[2,3]]);
        res=map(atleast_3d,[a,b])
        desired = [a[:,:,NewAxis],b[:,:,NewAxis]]
        assert_array_equal(res,desired)
    def check_3D_array(self):
        a = array([[1,2],[1,2]]); b = array([[2,3],[2,3]]);
        a = array([a,a]);b = array([b,b]);
        res=map(atleast_3d,[a,b])
        desired = [a,b]
        assert_array_equal(res,desired)

class test_hstack(unittest.TestCase):
    def check_0D_array(self):
        a = array(1); b = array(2);
        res=hstack([a,b])
        desired = array([1,2])
        assert_array_equal(res,desired)
    def check_1D_array(self):
        a = array([1]); b = array([2]);
        res=hstack([a,b])
        desired = array([1,2])
        assert_array_equal(res,desired)
    def check_2D_array(self):
        a = array([[1],[2]]); b = array([[1],[2]]);
        res=hstack([a,b])
        desired = array([[1,1],[2,2]])
        assert_array_equal(res,desired)

class test_vstack(unittest.TestCase):
    def check_0D_array(self):
        a = array(1); b = array(2);
        res=vstack([a,b])
        desired = array([[1],[2]])
        assert_array_equal(res,desired)
    def check_1D_array(self):
        a = array([1]); b = array([2]);
        res=vstack([a,b])
        desired = array([[1],[2]])
        assert_array_equal(res,desired)
    def check_2D_array(self):
        a = array([[1],[2]]); b = array([[1],[2]]);
        res=vstack([a,b])
        desired = array([[1],[2],[1],[2]])
        assert_array_equal(res,desired)
    def check_2D_array2(self):
        a = array([1,2]); b = array([1,2]);
        res=vstack([a,b])
        desired = array([[1,2],[1,2]])
        assert_array_equal(res,desired)

class test_dstack(unittest.TestCase):
    def check_0D_array(self):
        a = array(1); b = array(2);
        res=dstack([a,b])
        desired = array([[[1,2]]])
        assert_array_equal(res,desired)
    def check_1D_array(self):
        a = array([1]); b = array([2]);
        res=dstack([a,b])
        desired = array([[[1,2]]])
        assert_array_equal(res,desired)
    def check_2D_array(self):
        a = array([[1],[2]]); b = array([[1],[2]]);
        res=dstack([a,b])
        desired = array([[[1,1]],[[2,2,]]])
        assert_array_equal(res,desired)
    def check_2D_array2(self):
        a = array([1,2]); b = array([1,2]);
        res=dstack([a,b])
        desired = array([[[1,1],[2,2]]])
        assert_array_equal(res,desired)

""" array_split has more comprehensive test of splitting.
    only do simple test on hsplit, vsplit, and dsplit
"""
class test_hsplit(unittest.TestCase):
    """ only testing for integer splits.
    """
    def check_0D_array(self):
        a= array(1)
        try:
            hsplit(a,2)
            assert(0)
        except ValueError:
            pass
    def check_1D_array(self):
        a= array([1,2,3,4])
        res = hsplit(a,2)
        desired = [array([1,2]),array([3,4])]
        compare_results(res,desired)
    def check_2D_array(self):
        a= array([[1,2,3,4],
                  [1,2,3,4]])
        res = hsplit(a,2)
        desired = [array([[1,2],[1,2]]),array([[3,4],[3,4]])]
        compare_results(res,desired)

class test_vsplit(unittest.TestCase):
    """ only testing for integer splits.
    """
    def check_1D_array(self):
        a= array([1,2,3,4])
        try:
            vsplit(a,2)
            assert(0)
        except ValueError:
            pass
    def check_2D_array(self):
        a= array([[1,2,3,4],
                  [1,2,3,4]])
        res = vsplit(a,2)
        desired = [array([[1,2,3,4]]),array([[1,2,3,4]])]
        compare_results(res,desired)

class test_dsplit(unittest.TestCase):
    """ only testing for integer splits.
    """
    def check_2D_array(self):
        a= array([[1,2,3,4],
                  [1,2,3,4]])
        try:
            dsplit(a,2)
            assert(0)
        except ValueError:
            pass
    def check_3D_array(self):
        a= array([[[1,2,3,4],
                   [1,2,3,4]],
                  [[1,2,3,4],
                   [1,2,3,4]]])
        res = dsplit(a,2)
        desired = [array([[[1,2],[1,2]],[[1,2],[1,2]]]),
                   array([[[3,4],[3,4]],[[3,4],[3,4]]])]
        compare_results(res,desired)

class test_squeeze(unittest.TestCase):
    def check_basic(self):
        a = rand(20,10,10,1,1)
        b = rand(20,1,10,1,20)
        c = rand(1,1,20,10)
        assert_array_equal(squeeze(a),reshape(a,(20,10,10)))
        assert_array_equal(squeeze(b),reshape(b,(20,10,20)))
        assert_array_equal(squeeze(c),reshape(c,(20,10)))
                
# Utility

def compare_results(res,desired):
    for i in range(len(desired)):
        assert_array_equal(res[i],desired[i])


if __name__ == "__main__":
    ScipyTest('scipy_base.shape_base').run()




#
# io - Data input and output
#

from info_io import __doc__

from numpyio import packbits, unpackbits, bswap, fread, fwrite, \
     convert_objectarray
from mio import *
from array_import import *
from data_store import *
from pickler import *

from mmio import mminfo,mmread,mmwrite



# Copyright (c) 1996, 1997, The Regents of the University of California.
# All rights reserved.  See Legal.htm for full text and disclaimer.

from Numeric import *
from scipy_base.fastumath import *
from ezplot import *

def paws ( ) :
    i = raw_input ("Type in any string to continue; ^C to return to prompt. ")
    return

print "This program tests ezplot. Type demo () to begin the test."
paws ()

# ezcshow("false")
def demo ():
   import scipy
   rand = scipy.rand
   vsf = 0.
   c = 1
   s = 1000.
   kmax = 25
   lmax = 35
   # The following computations define an interesting 3d surface.
 
   xr = multiply.outer ( arange (1, kmax + 1), ones (lmax))
   yr = multiply.outer ( ones (kmax), arange (1, lmax + 1))
   zt = 5. + xr + .2 * rand (kmax, lmax)   # ranf (xr)
   rt = 100. + yr + .2 * rand (kmax, lmax)   # ranf (yr)
   z = s * (rt + zt)
   z = z + .02 * z * rand (kmax, lmax)   # ranf (z)
   ut = rt/sqrt (rt ** 2 + zt ** 2)
   vt = zt/sqrt (rt ** 2 + zt ** 2)
   ireg =  array(multiply.outer ( ones (kmax), ones (lmax)),'i')
   ireg [0:1, 0:lmax]=0
   ireg [0:kmax, 0:1]=0
   ireg [1:15, 7:12]=2
   ireg [1:15, 12:lmax]=3
   ireg [3:7, 3:7]=0
   freg=ireg + .2 * (1. - rand (kmax, lmax))  # ranf (ireg))
   freg=freg.astype (Float)
   #rt [4:6, 4:6] = -1.e8
   z [3:10, 3:12] = z [3:10, 3:12] * .9
   z [5, 5] = z [5, 5] * .9
   z [17:22, 15:18] = z [17:22, 15:18] * 1.2
   z [16, 16] = z [16, 16] * 1.1
   cgm("close")
   win("on")
   set_mesh(rt=rt,zt=zt,ireg=ireg)
   titleb("An interesting mesh")
   plotm()
   nf()
   paws()
   titleb("Plot with ranges")
   plotm(kstyle="dashed", lstyle = "dotted", krange = (0, 20), lrange = (0,10),
      color = "yellow", thick = 2)
   nf ( )
   paws()
   titleb("L lines only")
   plotb ( )
   plotm(kstyle="none", lstyle="dashed", color="orange")
   nf ( )
   paws ()
   k2 = 3
   l2 = 7
   plotb ( )
   titleb("Region 2 singled out")
   plotm (region = 2, kstyle = "dashed", lstyle = "dotted", color = "green")
   text("Region 2", zt [k2, l2], rt [k2, l2], 32.)
   nf ()
   paws ( )
   titlet("A different contour plot")
   plotc(pvar = z, lev = 20)
   nf ()
   paws ( )
   titlet("Same plot filled.")
   plotc(pvar = z, lev = 20, color = "fill")
   nf ()
   paws ( )
   clear_mesh ( )
   nz=20
   xs = array ( range (-nz-1, nz+1) )
   ys = xs
   zs = zeros ( (2*nz + 2, 2*nz + 2), Float)
   for i in range ( len (xs) ) :
      for j in range ( len (ys)) :
         rs = sqrt ( xs [i] * xs[i] + ys [j] * ys [j] ) + 1.e-12
         zs [i, j] = sin (rs) / rs
   titleb("Sombrero, contour plot")
   plotc (pvar = zs, zt = ys, rt = xs, lev = 12)
   nf ( )
   paws ( )
   titleb("Sombrero, logarithmic contour plot")
   plotc (pvar = zs, zt = ys, rt = xs, lev = -12)
   nf ( )
   paws ( )
   titleb("Sombrero, filled contour plot")
   plotc (pvar = zs, zt = ys, rt = xs, lev = 12, color = "fill")
   nf ( )
   paws ( )
   attr (labels = "yes")
   titleb("Sombrero, contour plot using plotz.")
   plotz (zs, xs, ys, lev = 12)
   nf ( )
   paws ( )
   attr (labels = "no")
   titleb("Sombrero, filled mesh, linear")
   plotf (zs, ys, xs)
   nf ( )
   paws ( )
   titleb("Sombrero, filled mesh, logarithmic")
   plotf (zs, ys, xs, cscale = "log")
   nf ( )
   paws ( )
   titleb("Sombrero, filled mesh, normal")
   plotf (zs, ys, xs, cscale = "normal")
   nf ( )
   paws ()
   nz = 80
   xs = array ( range (-nz-1, nz+1) )/4.
   ys = xs
   zs = zeros ( (2*nz + 2, 2*nz + 2), Float)
   for i in range ( len (xs) ) :
      for j in range ( len (ys)) :
         rs = sqrt ( xs [i] * xs[i] + ys [j] * ys [j] ) + 1.e-12
         zs [i, j] = sin (rs) / rs
   titleb("Sombrero, cell array plot")
   ploti (zs)
   nf ( )
   paws ( )
   # Vector stuff
   x = zeros ( (10, 10), Float)
   y = zeros ( (10, 10), Float)
   vx = zeros ( (10, 10), Float)
   vy = zeros ( (10, 10), Float)
   ireg = zeros ( (10, 10), Int)
   for i in range (10) :
      for j in range (10) :
         vy [i, j] = sin (i)
         vx [i, j] = cos (j)
         y [i, j] = j
         x [i, j] = i
   ireg [1:5, 1:5] = 1
   ireg [1:5, 5:10] = 2
   ireg [5:10, 1:5] = 3
   ireg [5:10, 5:10] = 4
   titleb ("A vector plot")
   plotv (y, x, vy, vx, ireg, vsc = 0.5, thick = 0.1, color = "red")
   nf ( )
   paws ( )
   set_mesh (rt = y, zt = x, ut = vy, vt = vx, ireg = ireg)
   titleb ("Vector plot, regions 1 and 4.")
   plotv (vsc = 0.35, region = [1,4], color = "blue")
   nf ( )
   paws ( )



""" General purpose classes for plotting utility.

    Many of these classes and function are helpful for laying out the 
    location of element of a graph (titles, tick labels, ticks, etc).
    A simple class for handling properties of graph elements is also
    included.  Everything here is independent of any specific GUI or
    drawing platform
    .
    The classes and functions break out into 3 main sections:
    
    General layout:
        class box_object   -- Helps align rectangular graph elements 
                              relative to one another.  Currently only
                              Int values are allowed for box dimensions.
        class point_object -- Alignment of points with one-another or 
                              box_objects.  
    Property Management:                          
        class poperty_object -- base class for any graph object that 
                                manages a set of properties.  Pretty
                                low-tech compared to graphite and Zope.
                                It could use additions for error checking.
                                
    Axis layout functions:
        auto_ticks          -- find set of tick locations for an axis.
        format_tick_labels  -- convert the numerical tick values to strings
                               for labels
        auto_interval       -- determine an appropriate tick interval for axis
        auto_bounds         -- determine appropriate end points for axis 
        and a few others...
"""


#----------------------------------------------------------
#               General layout classes
#----------------------------------------------------------
from Numeric import *
from scipy_base.fastumath import *
import scipy_base.limits as limits
import scipy_base as misc

LEFT,RIGHT,TOP,BOTTOM = 0,1,2,3 # used by same_as() method

TYPE = Int # all values in a box_object are forced to be integers
           # might change this later when we've looked at round off
           # issues more thoroughly.

class box_object:
    """ Helpful for laying out rectangles.
    
        This class has three general areas of functionality:
            1. Retreive information about a box such as it's size, 
               the x-coordinate of its left side, and others.
            2. Specify the location of one box (rectangle) in relation to 
               another using methods such as left_of(), above(), 
               or center_on().
            3. Change the size (and location) of a box by either choping off
               a portion of the box using trim_right(), trim_left(), and 
               others or inflating the box by a percentage.               
    """
    def __init__(self,topleft,size):
        self.topleft = asarray(topleft,TYPE)
        self._size = asarray(size,TYPE)

    #--------------- interrogation functions --------------------#
    def left(self): 
        "Return the x-coordinate of the left edge."
        return self.topleft[0]
    def right(self): 
        "Return the x-coordinate of the right edge."
        return self.topleft[0] + self.width()
    def top(self): 
        "Return the y-coordinate of the top edge."
        return self.topleft[1]
    def bottom(self):
        "Return the y-coordinate of the bottom edge." 
        return self.topleft[1] + self.height()
    def center_x(self): 
        "Return the x-coordinate of the boxes center."
        return self.topleft[0] + .5 * self.width()
    def center_y(self): 
        "Return the y-coordinate of the left edge."
        return self.topleft[1] + .5 * self.height()
    def size(self):
        "Return the size of the box as array((width,height))." 
        return self._size             
    def width(self):
        "Return the width of the box (length along x-axis)"
        return self.size()[0]    
    def height(self):
        "Return the width of the box (length along y-axis)"
        return self.size()[1]    
    def center(self):
        "Return the coordinates of the boxes center as an array"
        return self.topleft + .5 * self.size()
    def radial(self,angle):
        """ Length of a ray extending from the center of the box at the
            specified polar angle (in radians) to the edge of the box"""
        # Could have some div_by_zero errors if h or w = 0
        #   --> haven't thought about this much
        w,h = self.size()
        slope = abs(tan(angle))
        if slope > float(h)/w:
            s1 = h*.5
            s2 = s1/slope           
        else:            
            s1 = w*.5
            s2 = s1*slope            
        return sqrt(s1**2+s2**2)

    #------------- Moving and resizing methods ----------------#                
    def move(self,location):
        """ Move box so that its top left corner is located at
            the specified location.  Location is an (x,y) pair."""
        self.topleft = asarray(location,TYPE)        
    def translate(self,offset):
        """ Shift the box's location by the amount specifed in
            offset.  Offset is an (x,y) pair."""
        self.topleft = self.topleft + asarray(offset,TYPE)
    def set_size(self,sz):
        """ Set the size of the box. sz is an (x,y) pair."""
        self._size = asarray(sz,TYPE)
    def set_width(self,width):
        """ Set the width of the box. sz is a numeric value"""
        self.set_size((width,self.height()))
    def set_height(self,height):
        """ Set the height of the box. sz is a numeric value."""
        self.set_size((self.width(),height))

    #-------------------- Relational Positioning ----------------#        
    def above(self,other_box,margin=0):
        """Move box so that its bottom edge has same y-coord of the
           top edge of other_box.  The size does not change.  Optionally,
           margin specifies the gap between the boxes."""
        self.topleft[1] = other_box.top() - self.height() - margin
    def below(self,other_box,margin=0):
        """Move box so that its top edge has same y-coord of the 
           bottom edge of other_box.  The size does not change.  Optionally,
           margin specifies the gap between the boxes."""
        self.topleft[1] = other_box.bottom() + margin
    def left_of(self,other_box,margin=0):
        """Move box so that its right edge has same x-coord of the
           left edge of other_box.  The size does not change.  Optionally, 
           margin specifies the gap between the boxes."""
        self.topleft[0] = other_box.left() - self.width() - margin
    def right_of(self,other_box,margin=0):
        """Move box so that its left edge has same x-coord of the 
           right edge of other_box.  The size does not change.  Optionally,
           margin specifies the gap between the boxes."""
        self.topleft[0] = other_box.right() + margin
    def center_on_x_of(self,other_box): 
        """Move box so that x-coord of its center equals the x-coord of 
           other_box's center. The size does not change."""
        self.topleft[0] = other_box.center_x() - .5 * self.width()            
    def center_on_y_of(self,other_box):
        """Move box so that y-coord of its center equals the y-coord of 
           other_box's center. The size does not change."""
        self.topleft[1] = other_box.center_y() - .5 * self.height()            
    def center_on(self,other_box):
        """Move box so that its center is the same as other_box's center.
           The size does not change."""
        self.center_on_x_of(other_box)
        self.center_on_y_of(other_box)
    def same_as(self,other_box,edge,margin=0):
        """Set the specified edge of this box to be aligned with the
           same edge of other_box.  The size does not change.
        """   
        if edge == LEFT: 
            self.topleft[0] = other_box.left() + margin
        elif edge == RIGHT: 
            self.topleft[0] = other_box.right() - margin
        elif edge == TOP: 
            self.topleft[1] = other_box.top() + margin
        elif edge == BOTTOM: 
            self.topleft[1] = other_box.bottom() - margin                    
        else:
            raise ValueError, "edge should only be plt.xxx where xxx is" \
                              " LEFT,RIGHT,TOP, or BOTTOM"                              
    def radial_offset_from(self,other_box,angle,margin=0):
        """ Move this box so that its center is aligned along a radial ray
            out of other_box's center at the specified angle (in radians),
            and its edge touches the appropriate edge of other_box.
            Optionally, margin specifies the size of the gap (along the ray)
            between the two boxes.
        """ 
        dist = other_box.radial(angle) + margin + self.radial(angle)
        new_center = other_box.center() + rotate((dist,0),(0,0),angle)
        self.center_on(point_object(new_center.astype(TYPE)))
                
    #------------------ Trim the edges of the box ----------------#                 
    def trim_top(self,amount,margin=0):
        """ Remove amount from the top edge of the box.  The size and
            topleft corner of the box are altered appropriately.  Margin
            specifies an additional amount to be removed.
        """     
        self.topleft[1] = self.topleft[1] + amount
        self.set_height( self.height() - amount - margin)
    def trim_left(self,amount,margin=0):    
        """ Remove amount from the left edge of the box.  The size and
            topleft corner of the box are altered appropriately.  Margin
            specifies an additional amount to be removed.
        """     
        self.topleft[0] = self.topleft[0] + amount
        self.set_width( self.width() - amount - margin)
    def trim_right(self,amount,margin=0):    
        """ Remove amount from the right edge of the box.  The size and
            is altered appropriately.  Margin specifies an additional amount
            to be removed.
        """     
        self.set_width(self.width() - amount - margin)
    def trim_bottom(self,amount,margin=0):    
        """ Remove amount from the bottom edge of the box.  The size 
            is altered appropriately.  Margin specifies an additional 
            amount to be removed.
        """             
        self.set_height(self.height() - amount - margin)
    def trim_all(self,amount,margin=0):
        """ Remove amount from the all edges of the box.  The size and
            topleft corner of the bow are altered appropriately.  Margin
            specifies an additional amount to be removed.
        """     
        self.trim_top(amount,margin)
        self.trim_bottom(amount,margin)
        self.trim_left(amount,margin)
        self.trim_bottom(amount,margin)
    
    def inflate(self,percent):
        """ Expand the box in all directions by the specified percentage.
            Values < 1 shrink the box.  The center of the box remains the same.
        """
        old_size = self.size()
        inflated = old_size * percent
        shift = .5 * (old_size - inflated)
        self.topleft = floor(self.topleft + shift).astype(TYPE)
        self.set_size(floor(inflated).astype(TYPE))
    
    #------------------ test point relationships ----------------#
    def contains(self,pos):
        l,r,t,b = self.left(),self.right(),self.top(),self.bottom()
        #print self.text, self.size(), l,r,t,b
        #print pos[0],pos[1]
        if ((l <= pos[0] <= r) and (t <= pos[1] <= b)):
            return 1
        return 0
        
class point_object(box_object):
    """ Useful for laying points (and boxes) in relation to one another.
        
        point_object is a degenerate case of a box_object that has
        zero size.  The set_size() method is not allowed for points. As
        a result, size altering methods such as set_height and trimming
        functions are not valid.  Also, the radial() method always returns
        zero.  Other than that, the standard box_object methods work.
        point_objects can also be used as arguments to box_object methods.
    """
    def __init__(self,pt):
        box_object.__init__(self,pt,(0,0))        
    def radial(self,angle):        
        return 0        
    def set_size(self,sz):
        # this'll catch all set_size, trim_xxx and other inappropriate methods
        raise TypeError, "can't set the size of a point_object"

def bounding_points(objects):
    l = min(map(lambda x: x.left(),objects))
    t = min(map(lambda x: x.top(),objects))
    r = min(map(lambda x: x.right(),objects))
    b = min(map(lambda x: x.bottom(),objects))
    return (l,t),(r,b)

def bounding_box(objects):
    l = min(map(lambda x: x.left(),objects))
    t = min(map(lambda x: x.top(),objects))
    r = min(map(lambda x: x.right(),objects))
    b = min(map(lambda x: x.bottom(),objects))
    return box_object((l,t),(r-l,b-t))

#----------------------------------------------------------
#               Simple Property class
#----------------------------------------------------------

class property_object:
    """ Base class for graph object with properties like 'color', 'font', etc.
    
        Many object have a standard set of properties that describe the
        object.  Text objects, for example, have a color, font, and style.
        These attirbutes often have default values and perhaps only a range
        of acceptable values.  Every class derived from this one must have
        the variable "_attributes" defined at the class level.  The keys of
        this dictionary are the names of the class attributes.  The value
        for each key is a list with three entries.  The first is the default
        value for the attribute, the second is a list of acceptable values
        and the third is a short text description of the attribute.  FOr
        example:
          class text_object(property_class):
            _attributes = { 'color': ['black',['black','red','blue'],
                                      "The color of the text" ],
                            'style': ['normal',['normal','bold'],
                                      "The style of the text" ],}

        Currently only the first entry is used.  The second is rather limited
        in functionality, but might be useful for automatically generating
        dialog boxes.
        
        Graphite has a more flexible property system, but it is somewhat
        complex.  Zope also has a nice property structure.  If we run into
        major limitations with this 15 line implementation, we might look
        here for inspiration (or adoption...).
        
        There is no type safety enforced here which could cause so problems
        for unsuspecting users.  Maybe the optional typing of future Python
        versions will mitigate this.
    """    
    def __init__(self, kw, **attr):
        attrs = {};attrs.update(kw);
        if attr: attrs.update(attr)
        for name, value in self._attributes.items():
            val = value[0]
            try:
                val = attrs[name]
            except KeyError: pass
            self.__dict__[name] = val
    def reset_default(self):
        """ Reset the objects attributes to their default values.
        """
        for name, value in self._attributes.items():
            self.__dict__[name] = value[0]

    def clone_properties(self,other):
            """ Reset the objects attributes to their default values.
            """
            for name in other._attributes.keys():
                self.__dict__[name] = other.__dict__[name]

#----------------------------------------------------------#
#-------------- Axis and Tick mark utilities --------------#
#----------------------------------------------------------#

def rotate(pts,center,angle):
    """ pts is a Nx2 array of N (x,y) point pairs.  The points
        are rotated counter-clockwise around a center (x,y) point
        the specified angle (in radians).
    """
    t_pts = asarray(pts) - asarray(center)
    transform = array( ((cos(angle),-sin(angle)),
                        (sin(angle), cos(angle))) )
    rotated = transpose(dot(transform, transpose(t_pts)))
    t_pts = around(rotated + center)
    return t_pts.astype(TYPE)

def log2(num):
    """ Log base 2 of a number ( or array)
        
        !! 1e-16 is here to prevent errors when log is 0
    """
    if is_number(num) and num == 0: 
        num = num + 1e-16
    elif type(num) == type(array([])):
        putmask(num,equal(num,0.),1e-16)    
    return log10(num)/log10(2)

def is_base2(range):
    " True if value is base 2 (2, 4, 8, 16, ...)"
    l = log2(range)
    return (l == floor(l) and l > 0.0)

def is_number(val):
    " Returns 1 if value is an integer or double, 0 otherwise."
    return (type(val) in [type(0.0),type(0)]) 
    
default_bounds = ['auto','auto','auto']
def auto_ticks(data_bounds, bounds_info = default_bounds):
    """ Find locations for axis tick marks.
    
        Calculate the location for tick marks on an axis. data_bounds is a 
        sequence of 2 numbers specifying the maximum and minimum values of 
        the data along this axis. bounds_info is a sequence of 3 values that
        specify how the axis end points and tick interval are calculated. An
        array of tick mark locations is returned from the function.  The
        first and last tick entries are the axis end points.
        
        data_bounds -- (lower,upper).  The maximum and minimum values of the
                       data long this axis.  If any of the settings in 
                       bounds_info are 'auto' or 'fit', the axis properties
                       are calculated automatically from these settings.
        bounds_info -- (lower,upper,interval).  Each entry can either be
                       a numerical value or a string.  If a number,the axis 
                       property is set to that value.  If the entry
                       is 'auto', the property is calculated automatically.
                       lower and upper can also be 'fit' in which case
                       the axis end points are set equal to the values
                       in data_bounds.
    """
    # pretty ugly code...
    # man, this needs some testing.
    
    # hmmm, all the nan stuff should have been handled before this 
    # point...
    #from misc import nan_to_num, isinf
    #if is_number(bounds_info[0]): lower = nan_to_num(bounds_info[0])
    #else:                         lower = nan_to_num(data_bounds[0])
    #if is_number(bounds_info[1]): upper = nan_to_num(bounds_info[1])
    #else:                         upper = nan_to_num(data_bounds[1])
    #if is_number(bounds_info[2]): interval = nan_to_num(bounds_info[2])
    #else:                         interval = bounds_info[2]

    if is_number(bounds_info[0]): lower = bounds_info[0]
    else:                         lower = data_bounds[0]
    if is_number(bounds_info[1]): upper = bounds_info[1]
    else:                         upper = data_bounds[1]
    interval = bounds_info[2]

    #print 'raw input:', lower,upper,interval
    #print 'raw interval:', interval       
    if interval in ['linear','auto']:
        rng = abs(upper - lower)
        if rng == 0.:
            # anything more intelligent to do here?
            interval = .5
            lower,upper = data_bounds + array((-.5,.5))
        if is_base2(rng) and is_base2(upper) and rng > 4:
            if rng == 2: 
                interval = 1
            elif rng == 4: 
                interval = 4
            else: 
                interval = rng / 4 # maybe we want it 8
        else:
            interval = auto_interval((lower,upper))
    elif type(interval) in [type(0.0),type(0)]: 
        pass   
    else:
        #print 'interval: ', interval
        raise ValueError, interval + " is an unknown value for interval: " \
                          "  expects 'auto' or 'linear', or a number"
    
    # If the lower or upper bound are set to 'auto', 
    # calculate them based on the newly chosen interval.
    #print 'interval:', interval
    auto_lower,auto_upper = auto_bounds(data_bounds,interval)
    if bounds_info[0] == 'auto':
        lower = auto_lower
    if bounds_info[1] == 'auto':
        upper = auto_upper        
    
    # again, we shouldn't need this
    # cluge to handle inf values    
    #if isinf(lower):
    #    lower = nan_to_num(lower) / 10
    #if isinf(upper):
    #    upper = nan_to_num(upper) / 10    
    #if isinf(interval):
    #    interval = nan_to_num(interval) / 10    
        # if the lower and upper bound span 0, make sure ticks
    # will hit exactly on zero.
    
    if lower < 0 and upper > 0:
        #print 'arrrgh',0,upper+interval,interval
        hi_ticks = arange(0,upper+interval,interval)
        low_ticks = - arange(interval,-lower+interval,interval)        
        ticks = concatenate((low_ticks[::-1],hi_ticks))
    else:
        # othersize the ticks start and end on the lower and 
        # upper values.
        ticks = arange(lower,upper+interval,interval)
    #cluge
    if len(ticks) < 2:
        ticks = array(((lower-lower*1e-7),lower))
    #print 'ticks:',ticks
    if bounds_info[0] == 'fit': ticks[0] = lower
    if bounds_info[1] == 'fit': ticks[-1] = upper
    return ticks


def format_tick_labels(ticks):
    """ Convert tick values to formatted strings.
        Definitely needs some work
    """
    return map(str,ticks)        
    #print ticks
    #if equal(ticks,ticks.astype('l')):
    #    return map(str,ticks.astype('l'))
    #else:
    #    return map(str,ticks)        
    
        

def calc_bound(end_point,interval,end):
    """ Find an axis end point that includes the the value end_point.  If the
        tick mark interval results in a tick mark hitting directly on the 
        end_point, end_point is returned.  Otherwise, the location of the tick
        mark just past the end_point is returned. end is 'lower' or 'upper' to
        specify whether end_point is at the lower or upper end of the axis.
    """
    quotient,remainder = divmod(end_point,interval)    
    if not remainder: return end_point

    c1 = axis_bound = (quotient + 1) * interval
    c2 = axis_bound = (quotient) * interval
    if end == 'upper': return max(c1,c2)
    if end == 'lower': return min(c1,c2)                

    
def auto_bounds(data_bounds,interval):
    """ Calculate an appropriate upper and lower bounds for the axis from
        the the data_bounds (lower, upper) and the given axis interval.  The
        boundaries will either hit exactly on the lower and upper values
        or on the tick mark just beyond the lower and upper values.
    """    
    data_lower,data_upper = data_bounds
    lower = calc_bound(data_lower,interval,'lower')
    upper = calc_bound(data_upper,interval,'upper')
    return array((lower,upper))

def auto_interval(data_bounds):
    """ Calculate the tick intervals for a graph axis.

        Description:        
        The boundaries for the data to be plotted on the axis are:
            data_bounds = (lower,upper)
                
        A choice is made between 3 to 9 ticks marks (including end points)
        and tick intervals at 1, 2, 2.5, 5, 10, 20, ...
        
        Returns:
        interval -- float. tick mark interval for axis            
    """    
    range = float(data_bounds[1]) - float(data_bounds[0])
    # We'll choose from between 2 and 8 tick marks
    # Favortism is given to more ticks:
    #   Note reverse order and see CLUGE below...
    divisions = arange(8,2.,-1.) #(7,6,...,3)
    # Calculate the intervals for the divisions
    candidate_intervals = range / divisions
    # Get magnitudes and mantissas for each candidate
    magnitudes = 10.**floor(log10(candidate_intervals))
    mantissas = candidate_intervals / magnitudes
    
    # list of "pleasing" intervals between ticks on graph
    # only first magnitude listed, higher mags others inferred
    magic_intervals = array((1.,2.,2.5,5.,10.))
    # calculate the absolute differences between the candidates
    # (with mag removed) and the magic intervals
    differences = abs(magic_intervals[:,NewAxis] - mantissas) 

    # Find the division and magic interval combo
    # that produce the smallest differences.
    # also:
    # CLUGE: argsort doesn't preserve the order of
    # equal values, so we subtract a small , index
    # dependent amount from each difference to
    # force correct ordering
    sh = shape(differences)
    small = 2.2e-16 *arange(sh[1]) * arange(sh[0])[:,NewAxis]
    small = small[::-1,::-1] #reverse order
    differences = differences - small
    # ? Numeric should allow keyword "axis" ? comment out for now
    #best_mantissa = minimum.reduce(differences,axis=0)    
    #best_magic = minimum.reduce(differences,axis=-1)
    best_mantissa = minimum.reduce(differences,0)    
    best_magic = minimum.reduce(differences,-1)
    magic_index = argsort(best_magic)[0]
    mantissa_index = argsort(best_mantissa)[0]
    # The best interval is the magic_interval
    # multiplied by the magnitude of the best mantissa
    interval = magic_intervals[magic_index]
    magnitude = magnitudes[mantissa_index]
    #print differences
    #print 'results:', magic_index, mantissa_index,interval, magnitude
    #print 'returned:',interval*magnitude
    result = interval*magnitude
    if result == 0.0:
        result = limits.float_epsilon
    return result



#!/usr/bin/env python

import os
from scipy_distutils.core import Extension
from scipy_distutils.misc_util import get_path, default_config_dict, dot_join

def configuration(parent_package='',parent_path=None):
    package = 'signal'
    local_path = get_path(__name__,parent_path)    
    config = default_config_dict(package, parent_package)

    sources = ['sigtoolsmodule.c','firfilter.c','medianfilter.c']
    sources = [os.path.join(local_path,x) for x in sources]
    ext = Extension(dot_join(parent_package,package,'sigtools'), sources)
    config['ext_modules'].append(ext)
    
    sources = ['splinemodule.c','S_bspline_util.c','D_bspline_util.c',
               'C_bspline_util.c','Z_bspline_util.c','bspline_util.c']
    sources = [os.path.join(local_path,x) for x in sources]               
    ext = Extension(dot_join(parent_package,package,'spline'),sources)
    config['ext_modules'].append(ext)

    return config

if __name__ == '__main__':
    from scipy_distutils.core import setup
    setup(**configuration(parent_path=''))



""" Routines executed in main thread by gui_thread.

    o Starting up wxPython in secondary thread.

    o Shutting down the wxPython thread on system exit.

    o Generating proxy classes for wxPython window
      objects.
      
    This file does not contain an import of wxPython.
    Anything requiring the import of wxPython for its 
    definition should be placed in gui_thread_guts to 
    make sure the import occurs in the second thread.    
"""

###########################
# Secondary Thread Start-up
###########################
import thread, threading
import sys, os, new

running_in_second_thread = 0

# this variable flags if something is currently running inside a
# threading.Event.  If it is then other calls need not be proxied.
in_proxy_call = 0

gui_thread_finished = threading.Event()

def gui_thread(finished,extra=None):
    """ Indirectly imports wxPython into the second thread
    """
    try:
        # If we can find a module named wxPython.  Odds are (maybe 100%),
        # we don't want to start a new thread with a MainLoop() in it.
        if not sys.modules.has_key('wxPython'):
            #import must be done inside if statement!!!
            from gui_thread_guts import second_thread_app
            if extra is not None:
                exec( extra )
            # Variable used to see if the wxApp is
            # running in the main or secondary thread
            # Used to determine if proxies should be generated.            
            global running_in_second_thread,app,gui_thread_finished   
            app = second_thread_app(0)
            running_in_second_thread = 1
            finished.set()
            try:
                app.MainLoop()
                # when the main loop exits, we need to single the
                # exit_gui_thread function that it is OK to shut down.
            finally:
                gui_thread_finished.set()
    finally: 
        finished.set()             
        
def start(extra=None):                    
    finished = threading.Event()
    t1 = threading.Thread(target=gui_thread,args=(finished,))
    t1.setDaemon(1)
    t1.start()
    finished.wait()

def _start_up(src_modname,dst_modname,extra=None):
    start(extra)
    src_module = __import__(src_modname)
    for item in src_modname.split('.')[1:]:
        src_module = getattr(src_module,item)
    dst_module = sys.modules[dst_modname]
    for name in dir(src_module):
        if not hasattr(dst_module,name):
            setattr(dst_module,name,getattr(src_module,name))
    print '<Ready>\n>>> ',
    sys.stdout.flush()

def start_up(src_modname,dst_modname,extra=None):
    threading.Thread(target=_start_up,
                     args=(src_modname,dst_modname,extra)).start()
    print 'Wait for the <Ready> message and prompt!'
   
###########################
# Secondary Thread Clean-up
###########################
oldexitfunc = getattr(sys, 'exitfunc', None)
def exit_gui_thread(last_exit = oldexitfunc):    
    # don't wait on MS platforms -- it hangs.
    # On X11, we have to shut down the secondary thread.
    if running_in_second_thread and os.name != 'nt':
    	import gui_thread_guts
    	event_poster = gui_thread_guts.proxy_base()
    	event_catcher = event_poster.catcher
        if in_proxy_call:
            event_catcher.Close()
        else:
            finished = threading.Event()
            evt = gui_thread_guts.proxy_event(event_catcher.Close,
                                              (),{},finished)
            event_poster.post(evt)
            # wait for event to get handled
            finished.wait()
        # wait for the gui_thread to die.
        gui_thread_finished.wait()
    if last_exit: last_exit()
       
sys.exitfunc = exit_gui_thread        

##############################################
# Registering classes and generating proxies.
##############################################

class smart_class:
    
    """This class that creates a functor that returns either a proxied
    or non-proxied object depending on where it was 'called'.  If the
    call is made inside a proxied call then a non-proxied instance is
    returned, if not it returns a proxied instance.  This makes it
    possible to instantiate gui_threaded classes inside other
    gui_threaded objects without problems."""
    
    def __init__(self, orig_class, proxy_class):
        self.orig_class = orig_class
        self.proxy_class = proxy_class

    def __call__(self, *args, **kw):
        if in_proxy_call:
            return apply(self.orig_class, args, kw)
        else:
            return apply(self.proxy_class, args, kw)


def register(wx_class):
    """ Create a gui_thread compatible version of wx_class
    
        Test whether a proxy is necessary.  If so,
        generate and return the proxy class.  if not,
        just return the wx_class unaltered.   The proxied
        class is wrapped by the smart_class that returns
        a proxied or normal instance depending on where
        the instantiation occurs.
        
    """
    if running_in_second_thread:
        #print 'proxy generated'
        return smart_class(wx_class, proxify(wx_class))
    else:
        if not hasattr(wx_class, '_iNiT2'):
            if hasattr(wx_class, '__init__'):
                wx_class._iNiT2 = wx_class.__init__
            else:
                wx_class._iNiT2 = None
            wx_class.__init__ = plain_class__init__
        return wx_class

def plain_class__init__(self,*args,**kw):
    """This is apparently useful for apps like PyCrust."""
    if self._iNiT2:
        self._iNiT2(*args,**kw)
    add_close_event_handler(self)
    self.proxy_object_alive = 1
    
def proxify(wx_class):
    """ Create a proxy class for a wx_class.
    
        It searches for all the unique methods in wx_class
        and its bases classes and creates proxies for
        these calls.  The proxies fire events that
        are handled by the real wx_class object living
        in the wxPython thread.        
    """
    # generate proxy
    from gui_thread_guts import proxy_base
    class_name = 'proxy_'+wx_class.__name__
    class_bases = (proxy_base,)
    class_dict = {}
    class_doc = ""
    try:
        class_doc = getattr(wx_class, '__doc__')
    except AttributeError:
        pass
    else:
        class_dict['__doc__'] = class_doc
    class_methods = get_all_methods(wx_class)
    if class_methods.count('__init__') == 0:
        class_methods.insert(0, '__init__')
    for method in class_methods:
        func = generate_method(method,wx_class)
        if func:
            class_dict[method] = func
    proxy_class = new.classobj(class_name,class_bases,class_dict)
    return proxy_class

def add_close_event_handler(proxy_obj):
    """ Add an event handler to the wxPython object being wrapped by proxy_obj
        that temporarily catches the Close event.  This is necessary to 
        notify the proxy_obj when the wxPython object is dead.
    """
        
    import gui_thread_guts
    if hasattr(proxy_obj, 'PushEventHandler'):
        close_handler = gui_thread_guts.CloseEvtHandler(proxy_obj)
        try:
            proxy_obj.wx_obj.PushEventHandler(close_handler)
        except AttributeError:
            # its a standard class that just needs to let us
            # know when it dies.
            proxy_obj.PushEventHandler(close_handler)
        
def generate_method(method,wx_class):
    """ Create a proxy method.
    
        This first creates a text version of the method, 
        accounting for slight differences between __init__
        methods and all other methods.  It converts the
        text to a code object (using exec) and returns the
        code object.  The code is never actually written to
        a file.  
        
        It takes about .4 seconds on a wxFrame object with
        150 methods.  This is a one time cost at start up.
        It might be beneficial if we use the same code over
        and over to actually write the proxy class to a 
        module. (.pyc file?)        
    """
    module_name = wx_class.__module__
    class_name = wx_class.__name__
    import_statement = 'from %s import %s' % (module_name,class_name)    
    documentation = ""
    try:
        documentation = getattr(getattr(wx_class, method), '__doc__')
    except AttributeError:
        pass
    if method == '__init__':
        call_method = class_name
        pre_test = ''
        #pre_test = 'from gui_thread_guts import proxy_base;'\
        #           'proxy_base.__init__(self)'
        arguments = 'arg_list = args'
        results  = 'self.__dict__["wx_obj"] = ret_val;' \
                   'add_close_event_handler(self);' \
                   'self.__dict__["proxy_object_alive"] = 1;'
    elif (method == '__getattr__') or (method == '__del__') \
         or (method == '__setattr__'):
        return None
    else:
        pre_test =  "if not self.proxy_object_alive: proxy_error()"
        call_method = '%s.%s' % (class_name,method)
        arguments = 'arg_list = tuple([self.wx_obj] + list(args))'
        results  = 'return smart_return(ret_val)'
    body = """def %(method)s(self,*args,**kw):
                \"\"\"%(documentation)s\"\"\"
                %(pre_test)s
                from gui_thread_guts import proxy_event, smart_return
                %(import_statement)s
                # remove proxies if present
                args = dereference_arglist(args)
                dkw = dereference_dict(kw)
                %(arguments)s # inserts proxied object up front
                ret_val = None
                if in_proxy_call:
                    ret_val = apply(%(call_method)s, arg_list, dkw)
                else:
                    finished = threading.Event()
                    evt = proxy_event(%(call_method)s,arg_list,dkw,finished)
                    self.post(evt)
                    finished.wait()
                    if finished.exception_info:
                        raise finished.exception_info[0], \
                              finished.exception_info[1]
                    ret_val = finished._result
                %(results)s #results\n""" %locals()
    #if method == '__init__':
    #    print body
    exec(body)
    return eval(method)

#---------------------- utility functions ----------------------
import types
def is_method(x,cls):
    return type(cls.__dict__[x]) == types.FunctionType
    
def class_methods(cls):
    contents = dir(cls)
    methods = filter(lambda x,cls=cls: callable(getattr(cls, x)), contents)
    # using callable since it will handle more than just functions,
    # using getattr since it works for vtk objects too
    # (cls.__dict__[x]) won't work for vtk objects
    return methods

def get_all_methods(wx_class,top=1):
    assert (wx_class) is types.ClassType or hasattr(wx_class, '__bases__')
    methods = []
    for base in wx_class.__bases__:
        methods.extend(get_all_methods(base,0))
    methods.extend(class_methods(wx_class))    
    if top:
        methods = remove_duplicates(methods)
    return methods        

def remove_duplicates(lst):
    """ Discard any methods that have equivalent names
    """
    res = []    
    for x in lst:
        if not x in res: 
            res.append(x)
    return res

def is_proxy(x):
    return hasattr(x,'is_proxy')

def is_proxy_attr(x):
    return hasattr(x, 'x._proxy_attr__dont_mess_with_me_unless_you_know_what_youre_doing')

def get_proxy_attr_obj(x):
    return x._proxy_attr__dont_mess_with_me_unless_you_know_what_youre_doing

def dereference_obj(arg):
    """ Scan if object is proxied and convert to underlying object
    """
    if is_proxy(arg): 
        return arg.wx_obj
    elif is_proxy_attr(arg):
        obj = get_proxy_attr_obj(arg)
        if is_proxy(obj):
            # it is possible that an attribute is a proxy instance.
            return obj.wx_obj
        else:
            return obj
    else:
        return arg

def dereference_arglist(lst):
    """ Scans list for proxy objects and convert to underlying object
    """
    res = []
    for arg in lst:
        res.append(dereference_obj(arg))
    return res
       
def dereference_dict(kw):
    """ Scans dict for proxy objects and convert to underlying object
    """
    res = {}
    for key in kw.keys():
        arg = kw[key]
        res[key] = dereference_obj(arg)
    return res

def proxy_error():
    raise ValueError, 'This window has been destroyed'



#!/usr/bin/env python
#  $Id: wxtest.py,v 1.1 2003/09/30 17:32:40 travo Exp $
#  --------------------------------------------------------------------
#
#  NAME:     wxtest.py
#
#  PURPOSE:  Test interaction when combining wxPython and PyGist
#
#  NOTES:
#  Dave Grote (LBNL) was able to develop a GUI with wxWindows and PyGist
#  because wxWindows has an idle event.  Because in wxPython, the idle
#  event is only called once each time the window goes idle, Dave came
#  up with a workaround available in wxPython (to request that another
#  idle event be sent).  His solution is illustrated below.
#  
#  While we can get the GUI and the Gist window to each respond to 
#  events, the solution is not perfect.  If the GUI window covers the 
#  Gist window, the Gist window is not redrawn when exposed until the
#  mouse enters GUI window. 
#
#  CHANGES:
#  03/06/03 llc Originated.
#  --------------------------------------------------------------------

from wxPython.wx import *
import sys
import gist

ID_GIST = wxNewId()
ID_QUIT = wxNewId()

class MainFrame (wxFrame):
   def __init__ ( self, parent, id ):
      wxFrame.__init__ ( self, parent, id, "Idle Test" )
      wxButton ( self, ID_GIST, "Gist", pos=(0,0) )
      wxButton ( self, ID_QUIT, "Quit", pos=(0,50) )
      EVT_BUTTON ( self, ID_GIST, self.onGist )
      EVT_BUTTON ( self, ID_QUIT, self.onQuit )
      EVT_IDLE ( self, self.onIdle )
      self.isGistWindowOn = 0
   def onGist ( self, event ):
#     print "... Gist button pressed"
      self.isGistWindowOn = 1
      gist.plg([0,1]) 
   def onQuit ( self, event ):
#     print "... Quit button pressed"
      self.isGistWindowOn = 0
      sys.exit()
   def onIdle ( self, event ):
#     print "... Idle event"
      if self.isGistWindowOn:
         gist.pyg_pending() # Handle any new events
         gist.pyg_idler()   # Redraw window after changes from events
         event.RequestMore (1)

class MainApp ( wxApp ):
   def OnInit ( self ):
      self.frame = MainFrame ( NULL, -1 )
      self.frame.Show ( true )
      self.SetTopWindow ( self.frame )
      return true

if (__name__=="__main__"):
   app = MainApp (0)
   app.MainLoop()
